#!/bin/bash
#
#  PDF-Doppel-Seiten halbieren: Seiten senkrecht durchschneiden in der Seiten-Mitte 
#                               und als Einzel-Seiten-Dokument wieder zusammenfuegen
#
#  (c) 2012 Copyright <kurt.pfeifle@gmail.com>
#
#  Lizenz:  Public Domain, frei verwendbar, Aenderungen erlaubt.
#           Benutzung auf eigene Gefahr!
# 
#  Vorraussetzungen: (1) Linux, Mac OS X oder Solaris als Betriebssystem
#                    (2) Installation von Ghostscript und Poppler (oder XPDF-Tools)
#
#  Einschraenkungen: Skript rechnet nur mit Ganz-Zahlen. Dadurch kommt es zu kleineren
#                   Ungenauigkeiten bei Seitengroessen und Seitenverschiebungen.
#
#  (Mehr Details beim Aufruf des Skripts ohne Parameter.)


input="${1}"
result="einzelseiten-halbiert+umsortiert-$(basename "${input}")"
LANG=C
#LANG_ALL="UTF-8"

function echo_do() 
{
	echo
	echo "Kommando:    ${*}"
	echo "---------"
	echo
	"${@}"
}


function stringinstring() 
{
        case x"${2}" in x*${1}*)
                return 0 ;;
        esac
        return 1
}


function check_parameters() 
{
	case $# in
		0) usage
		   exit 0
		;;
		1) check_input_sanity
		   #exit 0
		;;
		*) echo " Mehr als 1 Datei angegeben. Verarbeite nur die erste, ${input}. Ignoriere den Rest."
		   check_input_sanity
		   #exit 0
		;;
	esac
}


function check_input_sanity() 
{
	if [ ! -e "${input}" ]; then
		echo
		echo " Es gibt keine Datei \"${input}\", oder Leseberechtigung fehlt."	
		echo
		exit 1
	fi

	mimeresult=$(file -Lbi "${input}")

	if stringinstring "application/pdf" "${mimeresult}"; then
		echo -n 
	else
		echo
		echo " FEHLER! Input ist keine PDF-Datei. Check des MIME-Types besagt:"
		echo
		echo "         ${mimeresult}"
		echo
		exit 1
	fi

	touch ${$}_${$}_touchtest
	if [ ! -e ${$}_${$}_touchtest ]; then
		echo
		echo " Werde Ergebnis-Datei '${result}' nicht speichern koennen -- Schreibberechtigung fehlt."	
		echo
		exit 1
	fi
	rm -rf  ${$}_${$}_touchtest

	# Anmerkung: unten stehende Pruefung dieses Skripts auf Umlaute funktioniert auf Mac OS nicht.
	case $(echo  "${input}") in
		*\ä*|*\ö*|*\ü*|*\Ä*|*\Ö*|*\Ü*|*\ß*)
			echo
			echo " Name der Input-Datei '$(basename "${input}")' enthaelt Umlaute, Leer- oder Sonderzeichen."
			echo " Bitte Datei umbenennen und erneut versuchen."
			echo
			exit 1
		;;
		*,*|*[*|*]*|*{*|*}*|*\ *)
			echo
			echo " Name der Input-Datei '$(basename "${input}")' enthaelt Umlaute, Leer- oder Sonderzeichen."
			echo " Bitte Datei umbenennen und erneut versuchen."
			echo
			exit 1
		;;
	esac
}


function ascii_art() 
{
	echo '
      Dieses Skript zerlegt die Seiten von PDFs in linke und rechte Haelften. Anwendungsfall ist:
   aus gescannten Doppelseiten wieder Einzelseiten machen (z.B. bei gescannten Broschueren).

      Das Schema zeigt die Anordnung gescannter Broschueren-Seiten. Auf realem Papier gedruckt, 
   gehoerten diese Seitenbilder selbstverstaendlich je paarweise zu 1 Druckbogen, der vorne und
   hinten bedruckt war. Die Boegen wurden gefaltet, zusammengelegt und per Rueckenstich geheftet. 

      Nach dem Entfernen der Heftklammern und dem Einscannen der jeweiligen Doppelseiten erhaelt
   man ein PDF mit der folgenden Seiten-Anordnung. Im konkreten Fall koennen sich natuerlich Ab-
   weichungen ergeben -- das haengt u.a. auch von der Scan-Reihenfolge ab. Im PDF stellt sich 
   die Seiten-Anordung logischerweise ohne das Konzept "doppelseitig bedruckt" dar -- hier sieht
   man jede Seite, als ob das Blatt nur 1-seitig bedruckt waere. Auf echtes Papier uebertragen,
   muesste man sich die Zeichnung so vorstellen, als ob jedes 2. Blatt nicht sichtbar waere
   (oder als Rueckseite zum jeweils vorherigen Blatt).
   
   Das vorliegende Skript stellt die vorliegende PDF-Seitengroessen automatisch fest. Es richtet
   sich nach den Dimensionen der erste Seite. Bei Abweichungen in den Folgeseiten (andere Seiten-
   groessen) gibt es eine Warnung aus, fuehrt seine Aktionen aber dennoch aus.

   Das Skript geht jedenfalls von folgendem Seiten-Schema aus:

 +----------------------------------------------------------------------------------------------------+
 |                                                                                                    |
 |                               +---------+---------+  ...........................Letzte Seite PDF   |
 |                               |linke    |   rechte|                                                |
 |                               |Mittel-  |  Mittel-|                                                |
 |                               |Seite    |    Seite|                                                |
 |                               |         |         |                                                |
 |             Innenblatt        |  +---------+---------+                                             |
 |             Broschuere - - -> +- |+---------+---------+                                            |
 |                                  ||+---------+---------+                                           |
 |                                  |||+---------+---------+                                          |
 |                                  ||||+---------+---------+                                         |
 |                                  |||||`        |`        |`                                        |
 |                                  +|||| `       | `       | `                                       |
 |                                   +|||  +---------+---------+   ................Sechste Seite PDF  |
 |                                    +||  |  6      |6.-letzte|                                      |
 |                                     +|  |  +---------+---------+   .............Fuenfte Seite PDF  |
 |                                      +--|  |5.-letzte|  5      |                                   |
 |                                       . |  |  +---------+---------+  ............Vierte Seite PDF  |
 |                                        .|  |  |  4      |4.-letzte|                                |
 |                                         +--|  |  +---------+---------+  .........Dritte Seite PDF  |
 |                                            |  |  |3.-letzte|  3      |                             |
 |  3. Blatt (Broschuere, Vorderseite) ---->  +--|  |  +---------+---------+  ......Zweite Seite PDF, |
 |                                               |  |  |  2      |vorletzte|          Umschlag innen. |
 |                                               +--|  |  +---------+---------+  ....Erste Seite PDF, |
 |                                                  |  |  |letzte   |         |      Umschlag aussen. |
 |  2. Blatt (Broschuere, Vorderseite) ---------->  +--|  |Seite    | Seite   |                       |
 |                                                     |  |         |         |                       |
 |                                                     +--|         |   1     |                       |
 |                                                        |         |         |                       |
 |  1. Blatt (Umschlag d. Broschuere, Aussenseite) ---->  +---------+---------+                       |
 |                                                                                                    |
 +----------------------------------------------------------------------------------------------------+
'
}


function usage() 
{
	echo
	echo " ======================:"
	echo " Hinweise zur Anwendung:"
	echo " ======================:"
	ascii_art
	echo
	echo "             Usage :  $(basename ${0})  INPUT-DATEI-NAME.pdf"
	echo
	echo "           Ergbnis :  einzelseiten-halbiert+umsortiert-INPUT-DATEI-NAME.pdf"
	echo "                      (wird abgelegt in $PWD ...)"
	echo
	echo "      Getestet mit :  Ghostscript v9.05 (fruehere Version funktionieren evtl. nicht!)"
	echo "               auf :  Ubuntu Oneiric und Mac OS X Lion."
	echo
	echo " bekanntes Problem :  Dateinamen mit Umlauten, Kommas, Leerzeichen oder nicht-ASCII-Zeichen funktionieren nicht."
	echo
}


function check_preconditions() 
{
	gs_return=$(which       gs      2>&1 >/dev/null ; echo ${?})
	pdfinfo_return=$(which  pdfinfo 2>&1 >/dev/null ; echo ${?})

	if [ "x${gs_return}" != "x0" ]; then
		echo
		echo " Keine Ghostscript-Installation gefunden."
		echo " Bitte Ghostscript installieren, um dieses Skript verwenden zu koennen."
		echo
		exit 1
	fi

	if [ "x${pdfinfo_return}" != "x0" ]; then
		echo
		echo " Keine 'pdfinfo'-Installation gefunden."
		echo " Bitte Poppler- oder XPDF-Tools installieren, um dieses Skript verwenden zu koennen."
		echo
		exit 1
	fi
}


function check_if_all_pages_have_same_size() 
{

	pages=$(pdfinfo  "${input}" | grep "Pages:" | awk '{print $2}')
	page_size_one=$(pdfinfo -f 1 -l        1 "${input}" | grep "Page.*size:" | awk '{print $4, $6}')
	page_size_all=$(pdfinfo -f 1 -l ${pages} "${input}" | grep "Page.*size:" | awk '{print $1, $2, $3, $4, $5, $6}')
	echo
	echo " Seitengroessen der PDF-Datei:"
	echo " ============================="
	echo "${page_size_all}" | sed 's#^#  #' | while read line ; do
		echo -en "$line" ; 
		if [ "x$(echo "${line}" | awk '{print $4, $6}')" != "x${page_size_one}" ]; then 
			echo "   (Verschieden von Seite 1)"; 
			touch ${$}_different
		else 
			echo ""; 
		fi 
		done

	if [ -e ${$}_different ]; then
		echo
		echo ' WARNUNG: Unterschiedliche Seitengroessen in der PDF-Datei!'
		echo "          (Manche) Folgeseiten haben andere Dimensionen als Seite 1."
		echo
		echo ' ----------------------------------------------------------------------.'
		echo ' ACHTUNG! Das Ergebnis wird eventuell nicht den Erwartungen entsprechen.'
		echo ' ACHTUNG! Das Ergebnis wird eventuell nicht den Erwartungen entsprechen.'
		echo ' ACHTUNG! Das Ergebnis wird eventuell nicht den Erwartungen entsprechen.'
		echo ' ----------------------------------------------------------------------.'
		echo
		echo
		rm -rf ${$}_different
	fi
}


function evaluate_dimension_first_page() 
{
	BxH=$(pdfinfo  "${input}" | grep "Page size:" | sed 's#Page size: *##;s# pt.*$##')
	if [ "x$(echo $(( ${pages} % 2 )))" == "x1" ]; then
		_odd=true
		echo
		echo  " Seitenzahl des PDF ist ${pages}."
		echo  " Eigentlich muesste eine gerade Anzahl vorliegen."
		echo  " Werde den Vorgang trotzdem fortsetzen..."
		sleep 5
		echo
	else
		_odd=false
	fi
        B=$(echo "${BxH}" | awk '{print $1}')
	B=$(echo "${B}  / 1" | bc)
        H=$(echo "${BxH}" | awk '{print $3}')
	H=$(echo "${H}  / 1" | bc)

	# "bc" liefert als Ergebnis nur Ganzzahlen ("bc -l" würde Nachkommastellen liefern).
	# Da wir die Ergebnisse als Anzal der Pixel verwenden wollen, und da Ghostscript
	# keine Bruchteile von Pixeln akzeptiert  :-)   ist dieses fuer uns Ok. Wir wollen
	# keine hoehere Praezision...
                Breite=$(echo "$(echo "${BxH}" | awk '{print $1}')      / 1" | bc )
              g_Breite=$(echo "$(echo "${BxH}" | awk '{print $1}') * 10 / 1" | bc )
          halbe_Breite=$(echo "$(echo "${BxH}" | awk '{print $1}')      / 2" | bc )
        g_halbe_Breite=$(echo "$(echo "${BxH}" | awk '{print $1}') * 10 / 2" | bc )
                 Hoehe=$(echo "$(echo "${BxH}" | awk '{print $3}')      / 1" | bc )
               g_Hoehe=$(echo "$(echo "${BxH}" | awk '{print $3}') * 10 / 1" | bc )
           halbe_Hoehe=$(echo "$(echo "${BxH}" | awk '{print $3}')      / 2" | bc )
         g_halbe_Hoehe=$(echo "$(echo "${BxH}" | awk '{print $3}') * 10 / 2" | bc )
	echo
	printf "%48s %10s\n"   "  Seiten-Dimensionen der 1. Seite (exakt) :  ${BxH}"
	echo
	echo                   " Im folgenden rechnet das Skript mit diesen Naeherungs-Werten:"
	echo                   " ------------------------------------------------------------"
	printf "%48s %10s\n"   "     Seitenbreite in PostScript-Punkten   :"  "$Breite"
	printf "%48s %10s\n"   "     Seitenbreite in Pixeln (bei 720 dpi) :"  "$g_Breite"
	printf "%48s %10s\n"   " 1/2 Seitenbreite in PostScript-Punkten   :"  "$halbe_Breite"
	printf "%48s %10s\n"   " 1/2 Seitenbreite in Pixeln (bei 720 dpi) :"  "$g_halbe_Breite"
	printf "%48s %10s\n"   "     Seitenhoehe  in PostScript-Punkten   :"  "$Hoehe"
	printf "%48s %10s\n"   "     Seitenhoehe  in Pixeln (bei 720 dpi) :"  "$g_Hoehe"
	printf "%48s %10s\n"   " 1/2 Seitenhoehe  in PostScript-Punkten   :"  "$halbe_Hoehe"
	printf "%48s %10s\n"   " 1/2 Seitenhoehe  in Pixeln (bei 720 dpi) :"  "$g_halbe_Hoehe"
	echo
	sleep 2
}


function ueberschuss_loeschen() 
{
	# Der Grund fuer diese Funktion ist: Ghostscript schreibt nach dem Seiten-Halbieren noch eine ueberfluessige leere (weisse) PDF-Seite.
        # Das liegt an dem PostScript-Schnipsel "-c <</PageOffset ....>>setpagedevice", den es auch noch der letzten PDF-Seite nochmals sieht,
	# weswegen es das PDF-Device erstmal initialisiert...
	surplus=$(echo $((${pages} + 1)))
	if [ "x$(ls -1 *"e_seitenhaelfte-$(basename "${input}")_"*"${surplus}.pdf" 1>/dev/null; echo ${?})" == "x0" ]; then
		echo
		echo "Ouch! Hier gibt's 'ne PDF-Seite zuviel...  Keine Sorge, das ist harmlos. Die Seite ist leer:"
		ls -l *"e_seitenhaelfte-$(basename "${input}")_${surplus}.pdf"
		echo "...deswegen loesche ich sie jetzt."
		echo_do rm -rf *"e_seitenhaelfte-$(basename "${input}")_${surplus}.pdf"
	fi
} 


function seitenhaelfte_machen() 
{

	echo -e " Erstelle PDFs mit den linken Seitenhaelften...  "
	echo_do  gs -q -o  "linke_seitenhaelfte-$(basename "${input}")_%d.pdf" -sDEVICE=pdfwrite -g${g_halbe_Breite}x${g_Hoehe} -c "<</PageOffset [            0    0]>> setpagedevice" -f "${input}"
	echo
	echo "Ergebnisse:"
	echo "-----------"
	ls -1 "linke_seitenhaelfte-$(basename "${input}")_"*.pdf | sed 's#^#  #'
	ueberschuss_loeschen
	echo " Fertig."

	echo
	echo -e " Erstelle PDFs mit den rechten Seitenhaelften...  "
	echo_do  gs -q -o "rechte_seitenhaelfte-$(basename "${input}")_%d.pdf" -sDEVICE=pdfwrite -g${g_halbe_Breite}x${g_Hoehe} -c "<</PageOffset [-${halbe_Breite} 0]>> setpagedevice" -f "${input}"
	echo
	echo "Ergebnisse:"
	echo "-----------"
	ls -1 "rechte_seitenhaelfte-$(basename "${input}")_"*.pdf | sed 's#^#  #'
	ueberschuss_loeschen
	echo " Fertig."
	echo
}


function sortierliste_erstellen() 
{
	echo " Neuzusammenbau der Seiten:"
	echo > ${$}---sortlist.txt   # ${$}---sortlist.txt leeren, falls sie bereits existiert
	case ${_odd} in
		true)
		    echo "    ACHTUNG, ungerade Anzahl von Input-Seiten wird Ergebnis 'komisch' aussehen lassen"
	
		    echo "Sortier-Reihenfolge ist:"
		    echo "------------------------"
		    for i in $( seq        0  2 ${pages} ); do echo "   linke_seitenhaelfte-$(basename "${input}")_${i}.pdf"; echo "  rechte_seitenhaelfte-$(basename "${input}")_$(( ${i} + 1 )).pdf"; done >> ${$}---sortlist.txt
		    for i in $( seq ${pages} -2        0 ); do echo "   linke_seitenhaelfte-$(basename "${input}")_${i}.pdf"; echo "  rechte_seitenhaelfte-$(basename "${input}")_$(( ${i} - 1 )).pdf"; done >> ${$}---sortlist.txt
		    _sortlist=$(cat ${$}---sortlist.txt | grep -v -- "_0.pdf")
		    echo "${_sortlist}"
		    #_sortlist=$(echo "${_sortlist}" | sed 's#^ *#"#; s#$#"#; s#""##')
		    #echo "${_sortlist}"
		    rm -rf ${$}---sortlist.txt
		    echo
		    echo "   NOCHMALS: ungerade Anzahl von Input-Seiten wird Ergebnis 'komisch' aussehen lassen"
		    echo
		;;
		false)
		    echo "Sortier-Reihenfolge ist:"
		    echo "------------------------"
		    for i in $( seq        1  2 ${pages} ); do echo "  rechte_seitenhaelfte-$(basename "${input}")_${i}.pdf"; echo "   linke_seitenhaelfte-$(basename "${input}")_$(( ${i} + 1 )).pdf"; done >> ${$}---sortlist.txt
		    for i in $( seq ${pages} -2        1 ); do echo "  rechte_seitenhaelfte-$(basename "${input}")_${i}.pdf"; echo "   linke_seitenhaelfte-$(basename "${input}")_$(( ${i} - 1 )).pdf"; done >> ${$}---sortlist.txt
		    _sortlist=$(cat ${$}---sortlist.txt | grep -v -- "_x.pdf")
		    echo "${_sortlist}"
		    #_sortlist=$(echo "${_sortlist}" | sed 's#^ *#"#; s#$#"#; s#""##')
		    #echo "${_sortlist}"
		    rm -rf ${$}---sortlist.txt
		;;
	esac
echo
echo pages=$pages
}


function zusammenbauen() 
{

	result="einzelseiten-halbiert+umsortiert-$(basename "${input}")"
	prepare_meta
	echo_do  gs -q -o "${result}" -sDEVICE=pdfwrite -c "[/Producer(${producer})/Keywords(${keywords})/DOCINFO pdfmark" -f ${_sortlist}
 
	echo
	echo -e "==================================================================================="
	echo -e " End-Ergebnis bitte auch visuell pruefen: '${result}' "
	echo -e "==================================================================================="
	echo
	pdfinfo "${result}"
	echo
}


function cleanup() 
{
	echo_do rm -rf rechte_seitenhaelfte-*.pdf linke_seitenhaelfte-*.pdf
}


function prepare_meta() 
{
	producer="$(pdfinfo "rechte_seitenhaelfte-$(basename "${input}")_1.pdf" | grep Producer: | sed 's#Producer: *##')"
	producer="${producer} [run from shellscript '$(basename ${0})', by <kurt.pfeifle@gmail.com>]"
	keywords="$(pdfinfo  "${input}"                                         | grep Keywords: | sed 's#Keywords: *##')"
	keywords="${keywords} # added: Heise-Verlag, iX Praxis | Tools und Tipps, Shellscript, Ghostscript & Co."
	#keywords="${keywords/^ #/#}"
	#producer=${producer/^ */}
}


check_preconditions

check_parameters "${@}"

check_if_all_pages_have_same_size

evaluate_dimension_first_page

seitenhaelfte_machen

sortierliste_erstellen

zusammenbauen

cleanup

#locale
